searchNodes=[{"ref":"Modular.html","title":"Modular","module":"Modular","type":"module","doc":"Apply modular programming principles and patterns to build better Elixir apps."},{"ref":"Modular.AreaAccess.html","title":"Modular.AreaAccess","module":"Modular.AreaAccess","type":"module","doc":"Ensures that module access is limited to given area&#39;s public contract. You want to divide your code into cohesive areas* that group common functionalities. These areas should provide external interfaces that act as contracts with clients and they should encapsulate implementation details inside their private parts so that it is impossible (and just unsuggested) for any of the clients to know about or depend upon these details. * We could replace the word &quot;area&quot; with &quot;bucket&quot;, &quot;context&quot;, &quot;component&quot;, &quot;package&quot;, &quot;assembly&quot; or... &quot;module&quot; - but not as in &quot;Elixir module defined via defmodule&quot; but rather as &quot;separate software unit that implements related functionality, expresses external interface and encapsulates implementation details&quot;. We&#39;ll stick to the word &quot;area&quot; though. In Elixir, we organize and encapsulate code into modules with defp as a tool for hiding the in-module private implementations. It&#39;s useful as long as the code fits into single module, but there&#39;s no means of access control across multiple modules (at least not yet). There&#39;s also a convention to differentiate modules as either public APIs or private implementations by filling the @moduledoc attribute with contract documentation or false respectively (as explained in official Elixir guidelines about writing docs). It&#39;s better than nothing, but as all modules are equally accessible and the meaning of @moduledoc is often misinterpreted, there&#39;s no out-of-the-box solution for implementing areas formed out of multiple modules and ensuring that no client will cross the boundary of area interface. This check aims to solve these problems by mapping dependencies among modules and verifying that the boundary of area is not abused. It applies the following ground rules: Areas are formed from a root module that has @moduledoc string filled and that acts as interface and from arbitrary number of implementation submodules with @moduledoc false. Area interfaces are globally accessible while area implementations are only accessible by modules within the same area (ie. that share the same area root module). Areas may be nested within other areas in which case they are also globally accessible and they are stil considered independent from their parent areas (ie. they can only reference parent interface and the parent can only reference their interface). Usage Include the check in your .credo.exs: %{ configs: [ %{ name: &quot;default&quot;, checks: [ {Modular.AreaAccess, ignore_callers: [~r/Test$/]} ] } ] } You can specify the following options: ignore_callers - all caller modules matching this regex (or list of regexes) will be ignored ignore_deps - all references to modules matching this regex (or list of regexes) will be ignored Example Let&#39;s consider the following application: defmodule Invoicing do @moduledoc &quot;Issues and manages client invoices.&quot; @doc &quot;Create new invoice with specified items.&quot; def create_invoice(items), do: __MODULE__.CreateInvoice.call(items) @doc &quot;Send specified invoice to specified e-mail address.&quot; def create_invoice(invoice_id, email), do: __MODULE__.SendInvoice.call(invoice_id, email) end defmodule Invoicing.Repo do @moduledoc false # ... end defmodule Invoicing.CreateInvoice do @moduledoc false def call(items) do invoice = Invoicing.Invoice.build(items) Invoicing.Repo.insert!(invoice) Invoicing.SendInvoice.call(invoice.id, &quot;invoices@backoffice.com&quot;) invoice end end defmodule Invoicing.SendInvoice do @moduledoc false def call(invoice_id, email), do: # ... end defmodule Invoicing.Invoice do @moduledoc &quot;Represents an issued invoice.&quot; defstruct [:id, :items, :number] def build(items) do %__MODULE__{ id: UUID.uuid4(), items: items, number: __MODULE__.GenerateNumber.call() } end end defmodule Invoicing.Invoice.GenerateNumber do @moduledoc false def call do # ... end end Here&#39;s how the check applies to the example above: Invoicing is the main publicly accessible area so it fills @moduledoc. Invoicing puts its implementation into private services CreateInvoice and SendInvoice. Invoicing returns Invoice to external clients therefore Invoice is also public. Invoice as separate area also has its own private service GenerateNumber. Invoicing can&#39;t directly use GenerateNumber and Invoice can&#39;t use eg. SendInvoice. Notes It&#39;s impossible to map all dependencies of modules in Elixir due to dynamic nature of the language (eg. the apply/3 function). Credo check is further limited by executing static AST analysis without compilation and only on a set of linted files. It&#39;s a best effort solution. Current implementation of this check follows idiomatic Elixir with a supoort for just one global level of publicity indicated via @moduledoc string - there&#39;s no concept of &quot;public within an app or within specific parent area&quot; although there are discussions about adding it to Elixir language. Right now, you must choose between eg. making app-wide modules public for sake of accessing them in nested areas or wrapping them in single larger area. This check ignores deps that have undefined publicity, so it&#39;s recommended to complement it with Credo&#39;s own Credo.Check.Readability.ModuleDoc in order to ensure that all modules are forced to define it via the @moduledoc attribute. In current implementation, usage of dependencies is not tracked down to specific points inside the caller module, but rather to its defmodule. This may be an inconvenience but it also reduces the number of duplicate issues for multiple references to the same dep within a single caller module. For unit testing purposes, access to private modules is allowed for callers with the same name, just with Test suffix appended. Of course, you may just as well decide to exclude all test files from the check by setting ignore_callers: [~r/Test$/]."},{"ref":"Modular.AreaAccess.html#base_priority/0","title":"Modular.AreaAccess.base_priority/0","module":"Modular.AreaAccess","type":"function","doc":"Returns the base priority for the check. Callback implementation for Credo.Check.base_priority/0."},{"ref":"Modular.AreaAccess.html#category/0","title":"Modular.AreaAccess.category/0","module":"Modular.AreaAccess","type":"function","doc":"Returns the category for the check. Callback implementation for Credo.Check.category/0."},{"ref":"Modular.AreaAccess.html#elixir_version/0","title":"Modular.AreaAccess.elixir_version/0","module":"Modular.AreaAccess","type":"function","doc":""},{"ref":"Modular.AreaAccess.html#explanation/0","title":"Modular.AreaAccess.explanation/0","module":"Modular.AreaAccess","type":"function","doc":"Callback implementation for Credo.Check.explanation/0."},{"ref":"Modular.AreaAccess.html#explanation_for_params/0","title":"Modular.AreaAccess.explanation_for_params/0","module":"Modular.AreaAccess","type":"function","doc":"Callback implementation for Credo.Check.explanation_for_params/0."},{"ref":"Modular.AreaAccess.html#format_issue/2","title":"Modular.AreaAccess.format_issue/2","module":"Modular.AreaAccess","type":"function","doc":"Callback implementation for Credo.Check.format_issue/2."},{"ref":"Modular.AreaAccess.html#params_defaults/0","title":"Modular.AreaAccess.params_defaults/0","module":"Modular.AreaAccess","type":"function","doc":""},{"ref":"Modular.AreaAccess.html#params_names/0","title":"Modular.AreaAccess.params_names/0","module":"Modular.AreaAccess","type":"function","doc":""},{"ref":"Modular.AreaAccess.html#run/3","title":"Modular.AreaAccess.run/3","module":"Modular.AreaAccess","type":"function","doc":""},{"ref":"Modular.AreaAccess.html#run_on_all?/0","title":"Modular.AreaAccess.run_on_all?/0","module":"Modular.AreaAccess","type":"function","doc":"Callback implementation for Credo.Check.run_on_all?/0."},{"ref":"Modular.ContractTests.html","title":"Modular.ContractTests","module":"Modular.ContractTests","type":"module","doc":"Ensures that public contracts are always covered with tests. Tests dedicated to contracts serve as living documentation for them. They present a complete sample usage along with prerequisites required for all functions to succeed. As such, they have a great advantage over textual documentation by always being up to date (as long as tests are being run as they should be). Besides that, as entry points for all the code inside their areas, testing them is a reasonable choice anyway for the purpose of validating the integration. The convention in Elixir is to name specific module&#39;s test the same as the module itself, just with the Test suffix and that&#39;s exactly the convention that this check verifies. Usage Include the check in your .credo.exs: %{ configs: [ %{ name: &quot;default&quot;, checks: [ {Modular.ContractTests, []} ] } ] } You can specify the following options: ignore_names - all modules matching this regex (or list of regexes) will be ignored Notes This check doesn&#39;t measure the test coverage, nor does it ensure that all public functions are indeed tested - it simply makes sure that developer looking for specific contract&#39;s test suite will have something to find. This check ignores modules that have undefined publicity, so it&#39;s recommended to complement it with Credo&#39;s own Credo.Check.Readability.ModuleDoc in order to ensure that all modules are forced to define it via the @moduledoc attribute. In current implementation, test module is required even for modules that don&#39;t provide any public functions (eg. structs). They should have a corresponding empty test modules for the check to pass. This helps to facilitate tests for structs, exceptions and DSLs that don&#39;t define functions but may need testing either way."},{"ref":"Modular.ContractTests.html#base_priority/0","title":"Modular.ContractTests.base_priority/0","module":"Modular.ContractTests","type":"function","doc":"Returns the base priority for the check. Callback implementation for Credo.Check.base_priority/0."},{"ref":"Modular.ContractTests.html#category/0","title":"Modular.ContractTests.category/0","module":"Modular.ContractTests","type":"function","doc":"Returns the category for the check. Callback implementation for Credo.Check.category/0."},{"ref":"Modular.ContractTests.html#elixir_version/0","title":"Modular.ContractTests.elixir_version/0","module":"Modular.ContractTests","type":"function","doc":""},{"ref":"Modular.ContractTests.html#explanation/0","title":"Modular.ContractTests.explanation/0","module":"Modular.ContractTests","type":"function","doc":"Callback implementation for Credo.Check.explanation/0."},{"ref":"Modular.ContractTests.html#explanation_for_params/0","title":"Modular.ContractTests.explanation_for_params/0","module":"Modular.ContractTests","type":"function","doc":"Callback implementation for Credo.Check.explanation_for_params/0."},{"ref":"Modular.ContractTests.html#format_issue/2","title":"Modular.ContractTests.format_issue/2","module":"Modular.ContractTests","type":"function","doc":"Callback implementation for Credo.Check.format_issue/2."},{"ref":"Modular.ContractTests.html#params_defaults/0","title":"Modular.ContractTests.params_defaults/0","module":"Modular.ContractTests","type":"function","doc":""},{"ref":"Modular.ContractTests.html#params_names/0","title":"Modular.ContractTests.params_names/0","module":"Modular.ContractTests","type":"function","doc":""},{"ref":"Modular.ContractTests.html#run/3","title":"Modular.ContractTests.run/3","module":"Modular.ContractTests","type":"function","doc":""},{"ref":"Modular.ContractTests.html#run_on_all?/0","title":"Modular.ContractTests.run_on_all?/0","module":"Modular.ContractTests","type":"function","doc":"Callback implementation for Credo.Check.run_on_all?/0."},{"ref":"Modular.Delegate.html","title":"Modular.Delegate","module":"Modular.Delegate","type":"module","doc":"Defines thin contracts with convention-driven call delegation to internal implementations."},{"ref":"Modular.Delegate.html#defcall/1","title":"Modular.Delegate.defcall/1","module":"Modular.Delegate","type":"macro","doc":""},{"ref":"Modular.Mutability.html","title":"Modular.Mutability","module":"Modular.Mutability","type":"module","doc":"Provides command/query annotations for functions within area contracts. Examples defmodule Invoicing do use Modular.Mutability @command true def create_user, do: :error @query true def get_users, do: [] end"},{"ref":"Modular.Owner.html","title":"Modular.Owner","module":"Modular.Owner","type":"module","doc":"Provides ownership annotations for area contracts. Examples defmodule Invoicing do use Modular.Owner @owner &quot;me@example.com&quot; end"},{"ref":"readme.html","title":"Modular","module":"readme","type":"extras","doc":"Modular Apply modular programming principles and patterns to build better Elixir apps. Modular is a toolbox that helps Elixir developers to apply modular programming principles and patterns in their projects. It includes checks for defining and enforcing modular programming rules &amp; conventions as well as utilities for mapping the characteristics of a growing project. Modular currently offers following tools: Modular.AreaAccess - ensures that module access is limited to given area&#39;s public contract Modular.ContractTests - ensures that public contracts are always covered with tests Modular.Delegate - defines thin contracts with call delegation to internal implementations Modular.Mutability - provides command/query annotations for functions within area contracts Modular.Owner - provides ownership annotations for area contracts"},{"ref":"readme.html#installation","title":"Modular - Installation","module":"readme","type":"extras","doc":"Add modular to your list of dependencies in mix.exs: def deps do [ {:modular, &quot;~&gt; 0.1&quot;} ] end"},{"ref":"changelog.html","title":"Changelog","module":"changelog","type":"extras","doc":"Changelog"},{"ref":"changelog.html#unreleased","title":"Changelog - Unreleased","module":"changelog","type":"extras","doc":"(add changes here)"},{"ref":"changelog.html#0-1-0","title":"Changelog - 0.1.0","module":"changelog","type":"extras","doc":"Initial versioned release"}]